"""XBMC/Kodi jsonclient library module."""
import json
import requests

# this list will be extended with types dynamically defined
__all__ = ["PLAYER_VIDEO",
           "KodiTransport",
           "KodiJsonTransport",
           "Kodi",
           "KodiNamespace", ]

# Kodi constant
PLAYER_VIDEO = 1

# Dynamic namespace class injection
__KODI_NAMESPACES__ = (
    "Addons", "Application", "AudioLibrary", "Favourites", "Files", "GUI",
    "Input", "JSONRPC", "Playlist", "Player", "PVR", "Settings", "System",
    "VideoLibrary", "xbmc")


class KodiTransport(object):
    """Base class for Kodi transport."""

    def execute(self, method, args):
        """Execute method with given args."""
        pass  # pragma: no cover


class KodiJsonTransport(KodiTransport):
    """HTTP Json transport."""

    def __init__(self, url, username='xbmc', password='xbmc'):
        self.url = url
        self.username = username
        self.password = password
        self._id = 0

    def execute(self, method, *args, **kwargs):
        headers = {
            'Content-Type': 'application/json',
            'User-Agent': 'python-kodi'
        }
        # Params are given as a dictionnary
        if len(args) == 1:
            args = args[0]
            params = kwargs
            # Use kwargs for param=value style
        else:
            args = kwargs
        params = {}
        params['jsonrpc'] = '2.0'
        params['id'] = self._id
        self._id += 1
        params['method'] = method
        params['params'] = args

        values = json.dumps(params)

        resp = requests.post(self.url,
                             values.encode('utf-8'),
                             headers=headers,
                             auth=(self.username, self.password))
        resp.raise_for_status()
        return resp.json()


class Kodi(object):
    """Kodi client."""

    def __init__(self, url, username='xbmc', password='xbmc'):
        self.transport = KodiJsonTransport(url, username, password)
        # Dynamic namespace class instanciation
        # we obtain class by looking up in globals
        _globals = globals()
        for cl in __KODI_NAMESPACES__:
            setattr(self, cl, _globals[cl](self.transport))

    def execute(self, *args, **kwargs):
        """Execute method with given args and kwargs."""
        self.transport.execute(*args, **kwargs)


class KodiNamespace(object):
    """Base class for Kodi namespace."""

    def __init__(self, kodi):
        self.kodi = kodi

    def __getattr__(self, name):
        klass = self.__class__.__name__
        method = name
        kodimethod = "%s.%s" % (klass, method)

        def hook(*args, **kwargs):
            """Hook for dynamic method definition."""
            return self.kodi.execute(kodimethod, *args, **kwargs)

        return hook

# inject new type in module locals
_LOCALS_ = locals()
for _classname in __KODI_NAMESPACES__:
    # define a new type extending KodiNamespace
    # equivalent to
    #
    # class Y(KodiNamespace):
    #    pass
    _LOCALS_[_classname] = type(_classname, (KodiNamespace, ), {})
    # inject class in __all__ for import * to work
    __all__.append(_classname)



"""Test for kodijson module"""
import json
import responses

# not used, ensure import is working
from kodijson import PLAYER_VIDEO  # NOQA

from kodijson import Kodi
from kodijson import Addons

from kodijson import Application
from kodijson import AudioLibrary

from kodijson import Favourites
from kodijson import Files
from kodijson import GUI
from kodijson import Input
from kodijson import JSONRPC

from kodijson import Playlist
from kodijson import Player
from kodijson import PVR

from kodijson import Settings
from kodijson import System
from kodijson import VideoLibrary
from kodijson import xbmc

from kodijson import KodiJsonTransport


class TestKodiJsonTransport(object):
    """ Tests for default transport """

    def test_default(self):
        """Tests KodiJsonTransport default values"""
        url = "http://localhost/"
        transport = KodiJsonTransport(url=url)
        assert transport.url == url
        assert transport.username == "xbmc"
        assert transport.password == "xbmc"

    def test_parameters(self):
        """Tests KodiJsonTransport default values"""

        url = "http://localhost/"
        transport = KodiJsonTransport(url=url, username="kodi", password="pwd")
        assert transport.url == url
        assert transport.username == "kodi"
        assert transport.password == "pwd"

    @responses.activate
    def test_http_call_no_args(self):
        """ Test http call with no arguments"""
        # the response does not matters
        responses.add("POST", "http://localhost", "{}")
        url = "http://localhost/"
        transport = KodiJsonTransport(url=url, username="kodi", password="pwd")
        # test
        transport.execute("remote_method")

        expectd_body = {
            "method": "remote_method",
            "id": 0,
            "jsonrpc": "2.0",
            "params": {}  # no param given
        }
        assert responses.calls[0].request.url == 'http://localhost/'
        assert responses.calls[0].request.method == 'POST'
        assert expectd_body == json.loads(
            responses.calls[0].request.body.decode("utf-8"))

    @responses.activate
    def test_http_call_kwargs(self):
        """ Test http call with kwargs arguments"""

        # the response does not matters
        responses.add("POST", "http://localhost", "{}")

        url = "http://localhost/"
        transport = KodiJsonTransport(url=url, username="kodi", password="pwd")
        # test
        transport.execute("remote_method", x="y", z="a")

        expectd_body = {
            "method": "remote_method",
            "id": 0,
            "jsonrpc": "2.0",
            "params": {'x': 'y',
                       'z': 'a'}
        }
        assert responses.calls[0].request.url == 'http://localhost/'
        assert responses.calls[0].request.method == 'POST'
        assert expectd_body == json.loads(
            responses.calls[0].request.body.decode("utf-8"))

    @responses.activate
    def test_http_call_args(self):
        """ Test http call with kwargs arguments"""

        # the response does not matters
        responses.add("POST", "http://localhost", "{}")
        url = "http://localhost/"
        transport = KodiJsonTransport(url=url, username="kodi", password="pwd")
        # test
        transport.execute("remote_method", {"x": "y"})

        expectd_body = {
            'id': 0,
            'jsonrpc': '2.0',
            'method': 'remote_method',
            "params": {'x': 'y'}
        }
        assert responses.calls[0].request.url == 'http://localhost/'
        assert responses.calls[0].request.method == 'POST'
        assert expectd_body == json.loads(
            responses.calls[0].request.body.decode("utf-8"))


class TestKodi(object):
    """ Tests for default Kodi class """
    def test_default(self):
        """Tests KodiJsonTransport default values"""

        url = "http://localhost/"
        x = Kodi(url=url)
        assert isinstance(x.Addons, Addons)

        assert isinstance(x.Application, Application)
        assert isinstance(x.AudioLibrary, AudioLibrary)

        assert isinstance(x.Favourites, Favourites)
        assert isinstance(x.Files, Files)
        assert isinstance(x.GUI, GUI)
        assert isinstance(x.Input, Input)
        assert isinstance(x.JSONRPC, JSONRPC)

        assert isinstance(x.Playlist, Playlist)
        assert isinstance(x.Player, Player)
        assert isinstance(x.PVR, PVR)

        assert isinstance(x.Settings, Settings)
        assert isinstance(x.System, System)
        assert isinstance(x.VideoLibrary, VideoLibrary)
        assert isinstance(x.xbmc, xbmc)

    @responses.activate
    def test_ping(self):
        """ Test a call with JSON.Ping """
        # the response does not matters
        responses.add("POST", "http://localhost", "{}")
        url = "http://localhost/"

        x = Kodi(url=url)
        x.JSONRPC.Ping()

        expectd_body = {
            'id': 0,
            'jsonrpc': '2.0',
            'method': 'JSONRPC.Ping',
            'params': {}
        }
        assert responses.calls[0].request.url == 'http://localhost/'
        assert responses.calls[0].request.method == 'POST'
        assert expectd_body == json.loads(
            responses.calls[0].request.body.decode("utf-8"))

    @responses.activate
    def test_execute_ping(self):
        """ Test a call with JSON.Ping """
        # the response does not matters
        responses.add("POST", "http://localhost", "{}")
        url = "http://localhost/"

        x = Kodi(url=url)
        x.execute("JSONRPC.Ping", x=1)

        expectd_body = {
            'id': 0,
            'jsonrpc': '2.0',
            'method': 'JSONRPC.Ping',
            'params': {"x": 1}
        }
        assert responses.calls[0].request.url == 'http://localhost/'
        assert responses.calls[0].request.method == 'POST'
        assert expectd_body == json.loads(
            responses.calls[0].request.body.decode("utf-8"))



from setuptools import setup
import version

PACKAGE = 'kodi-json'

setup(name=PACKAGE,
      version=version.VERSION,
      license="WTFPL",
      description="Python module for controlling kodi over HTTP Json API",
      author="Jean-Christophe Saad-Dupuy",
      author_email="jc.saaddupuy@fsfe.org",
      url="https://github.com/jcsaaddupuy/python-kodijson",
      py_modules=["kodijson/kodijson"],
      packages=["kodijson"],
      install_requires=["requests"],
      setup_requires=['pytest-runner'],
      tests_require=['pytest', 'pytest-cov', 'responses'],
      classifiers=[
          "Development Status :: 4 - Beta",
          "Topic :: Utilities",
          "Topic :: Multimedia",
          "Intended Audience :: Developers",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.1",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: 3.5",
      ])
